--------------------------------------------------------------------------------------------------------
--                                             Localized global                                       --
--------------------------------------------------------------------------------------------------------
local _G = getfenv(0)

--------------------------------------------------------------------------------------------------------
--                                             AceAddon init                                          --
--------------------------------------------------------------------------------------------------------
local MODNAME	= "Broker_BagInfo"
local addon = LibStub("AceAddon-3.0"):NewAddon(MODNAME, "AceEvent-3.0", "AceTimer-3.0")
_G.Broker_BagInfo = addon

local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local Crayon = LibStub:GetLibrary("LibCrayon-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(MODNAME)
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceDB = LibStub("AceDB-3.0")
local AceDBOptions = LibStub("AceDBOptions-3.0")


--------------------------------------------------------------------------------------------------------
--                                 Broker_BagInfo variables and defaults                              --
--------------------------------------------------------------------------------------------------------
local profileDB
local DATABASE_DEFAULTS = {
	profile = {
		countSpecial = false,
		bagsOnClick = 2,
		showFree = false,
		autoSell = false,
	},
}

local request = true


--------------------------------------------------------------------------------------------------------
--                                  Broker_BagInfo options panel                                      --
--------------------------------------------------------------------------------------------------------
addon.options = {
	type = "group",
	name = MODNAME,
	args = {
		general = {
			order = 1,
			type = "group",
			name = L["General Settings"],
			cmdInline = true,
			args = {
				separator1 = {
					type = "header",
					name = L["Display Options"],
					order = 1,
				},
				details = {
					order = 2,
					type = "toggle",
					name = L["Count special bags."],
					desc = L["Toggle to count professions and ammo bags."],
					get = function()
						return profileDB.countSpecial
					end,
					set = function(key, value)
						profileDB.countSpecial = value
						addon:UpdateText()
					end,
				},
				showFree = {
					order = 3,
					type = "toggle",
					name = L["Show free space."],
					desc = L["Toggle to show free space instead of used space."],
					get = function()
						return profileDB.showFree
					end,
					set = function(key, value)
						profileDB.showFree = value
						addon:UpdateText()
					end,
				},
				onClick = {
					order = 4,
					type = "select",
					name = L["On click:"],
					desc = L["Choose whether to open bags when you click the BagInfo icon."],
					get = function()
						return profileDB.bagsOnClick
					end,
					set = function(key, value)
						profileDB.bagsOnClick = value
					end,
					values = function()
						return {
							[0] = L["Do nothing"],
							[1] = L["Open backpack"],
							[2] = L["Open all bags"],
						}
					end,
				},
				separator2 = {
					type = "header",
					name = L["Auto Sell Options"],
					order = 5,
				},
				sell = {
					order = 6,
					type = "toggle",
					name = L["Auto sell"],
					desc = L["Toggle to sell gray items at vendor automatically."],
					get = function()
						return profileDB.autoSell
					end,
					set = function(key, value)
						profileDB.autoSell = value
					end,
				},
			},
		},
	},
}

function addon:SetupOptions()
	addon.options.args.profile = AceDBOptions:GetOptionsTable(self.db)
	addon.options.args.profile.order = -2

	AceConfig:RegisterOptionsTable(MODNAME, addon.options, nil)
	
	self.optionsFrames = {}
	self.optionsFrames.general = AceConfigDialog:AddToBlizOptions(MODNAME, nil, nil, "general")
	self.optionsFrames.profile = AceConfigDialog:AddToBlizOptions(MODNAME, L["Profiles"], MODNAME, "profile")
end


--------------------------------------------------------------------------------------------------------
--                                        Broker_BagInfo core                                         --
--------------------------------------------------------------------------------------------------------
function addon:OnInitialize()
	self.db = AceDB:New("Broker_BagInfoDB", DATABASE_DEFAULTS, true)
	if not self.db then
		print("Error: Database not loaded correctly.  Please exit out of WoW and delete Broker_BagInfo.lua found in: \\World of Warcraft\\WTF\\Account\\<Account Name>>\\SavedVariables\\")
	end
	
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	
	profileDB = self.db.profile
	self:SetupOptions()
	
	self:RegisterEvent("BAG_UPDATE","ScheduleUpdate")
	self:RegisterEvent("MERCHANT_SHOW","OnMerchantShow")
	
	self:MainUpdate()
end

function addon:OnEnable()
	self:ScheduleRepeatingTimer("MainUpdate", 1)
end

-- LDB object
addon.obj = ldb:NewDataObject(MODNAME, {
	type = "data source",
	label = "BagInfo",
	text = "0/0",
	icon = "Interface\\Icons\\INV_Misc_Bag_10",
	OnClick = function(frame, msg)
		if msg == "RightButton" then
			addon:ShowConfig()
		elseif msg == "LeftButton" then
			if profileDB.bagsOnClick == 1 then
				ToggleBackpack()
			elseif profileDB.bagsOnClick == 2 then
				OpenAllBags()
			end
		end
	end,
	OnTooltipShow = function(tooltip)
		if not tooltip or not tooltip.AddLine then return end
		tooltip:AddLine(MODNAME.." "..GetAddOnMetadata(MODNAME, "Version"))
		tooltip:AddLine(L["Left-hint"])
		tooltip:AddLine(L["Right-hint"])
	end,
})

function addon:MainUpdate()
	if (request) then
		self:UpdateText()
		request = false
	end
end

--------------------------------------------------------------------------------------------------------
--                                   Broker_BagInfo event handlers                                    --
--------------------------------------------------------------------------------------------------------
function addon:ScheduleUpdate()
	request = true
end

function addon:OnMerchantShow()
	if profileDB.autoSell then
		self:SellGrayItems()
	end
end

--------------------------------------------------------------------------------------------------------
--                                      Broker_BagInfo functions                                      --
--------------------------------------------------------------------------------------------------------

-- Called after profile changed
function addon:OnProfileChanged(event, database, newProfileKey)
	profileDB = database.profile
end

-- Open config window
function addon:ShowConfig()
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.profile)
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.general)
end

-- Text update function
function addon:UpdateText()
	local total, free = 0, 0
	
	for i = 0, NUM_BAG_SLOTS do
		local freeSlots, bagType = GetContainerNumFreeSlots(i)
		if bagType then
			if profileDB.countSpecial then
				total = total + GetContainerNumSlots(i)
				free = free + freeSlots
			elseif not (self:IsAmmoBag(bagType) or self:IsProfessionBag(bagType)) then
				total = total + GetContainerNumSlots(i)
				free = free + freeSlots
			end
		end
	end
	
	local taken = total - free
	if profileDB.showFree then
		self.obj.text = (string.format("|cff%s%d/%d|r", Crayon:GetThresholdHexColor(free / total), free, total))
	else
		self.obj.text = (string.format("|cff%s%d/%d|r", Crayon:GetThresholdHexColor(free / total), taken, total))
	end
end

function addon:IsAmmoBag(bagType)
	-- 4: Soul Bag
	-- 2: Ammo Pouch
	-- 1: Quiver
	if bagType == 4 or bagType == 2 or bagType == 1 then
		return true
	end
	return false
end

function addon:IsProfessionBag(bagType)
	-- 1024: Mining Bag
	-- 512: Gem Bag
	-- 128: Engineering Bag
	-- 64: Enchanting Bag
	-- 32: Herb Bag
	-- 16: Inscription Bag
	-- 8: Leatherworking Bag
	if bagType == 1024 or bagType == 512 or bagType == 128 or bagType == 64 or bagType == 32 or bagType == 16 or bagType == 8 then
		return true
	end
	return false
end

-- Show money with icons
function addon:CopperToString(c)
	local str = ""
	if not c or c < 0 then 
		return str 
	end
	
	if c >= 10000 then
		local g = math.floor(c/10000)
		c = c - g*10000
		str = str.."|cFFFFD800"..g.."|r |TInterface\\MoneyFrame\\UI-GoldIcon.blp:0:0:0:0|t"
	end
	if c >= 100 then
		local s = math.floor(c/100)
		c = c - s*100
		str = str.."|cFFC7C7C7"..s.."|r |TInterface\\MoneyFrame\\UI-SilverIcon.blp:0:0:0:0|t"
	end
	if c >= 0 then
		str = str.."|cFFEEA55F"..c.."|r |TInterface\\MoneyFrame\\UI-CopperIcon.blp:0:0:0:0|t"
	end
	
	return str
end

-- Sell gray items at vendor
function addon:SellGrayItems()
	for bag = 0, NUM_BAG_SLOTS do
		for slot = 1, GetContainerNumSlots(bag) do
			local id = GetContainerItemID(bag, slot)
			if id then
				local _, itemLink, itemRarity, _,  _, _, _, _, _, _, sellPrice = GetItemInfo(id)
				if itemRarity == 0 and sellPrice > 0 then
					local _, itemCount, locked = GetContainerItemInfo(bag, slot)
					if not locked then
						if itemCount == 1 then 
							DEFAULT_CHAT_FRAME:AddMessage(string.format("|cff00ff00[Broker_BagInfo]|r %s %s %s %s", L["Sold"], itemLink, L["for"], self:CopperToString(math.floor(sellPrice))))
						else
							DEFAULT_CHAT_FRAME:AddMessage(string.format("|cff00ff00[Broker_BagInfo]|r %s %dx%s %s %s", L["Sold"], itemCount, itemLink, L["for"], self:CopperToString(math.floor(sellPrice*itemCount))))
						end
						UseContainerItem(bag,slot)
					end
				end
			end
		end
	end
end
